// Copyright (C) 2015 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

#ifndef QGEOERROR_MESSAGES_H
#define QGEOERROR_MESSAGES_H

#include <qglobal.h>

QT_BEGIN_NAMESPACE

extern const char NOKIA_PLUGIN_CONTEXT_NAME[];
extern const char MISSED_CREDENTIALS[];
extern const char SAVING_PLACE_NOT_SUPPORTED[];
extern const char REMOVING_PLACE_NOT_SUPPORTED[];
extern const char SAVING_CATEGORY_NOT_SUPPORTED[];
extern const char REMOVING_CATEGORY_NOT_SUPPORTED[];
extern const char PARSE_ERROR[];
extern const char NETWORK_ERROR[];
extern const char CANCEL_ERROR[];
extern const char RESPONSE_NOT_RECOGNIZABLE[];

QT_END_NAMESPACE

#endif // QGEOERROR_MESSAGES_H
