// Copyright (C) 2017 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause

#include "requesthandler.h"

int main(int /*argc*/, char ** /*argv*/)
{
    return 0;
}

