// Copyright (C) 2015 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
#ifndef URI_CONSTANTS_H
#define URI_CONSTANTS_H

#include <QString>

QT_BEGIN_NAMESPACE

extern const QString ROUTING_HOST;
extern const QString GEOCODING_HOST;
extern const QString REVERSE_GEOCODING_HOST;
extern const QString PLACES_HOST;
extern const QString MAP_TILES_HOST;
extern const QString MAP_TILES_HOST_AERIAL;

QT_END_NAMESPACE

#endif // URI_CONSTANTS_H
