// Copyright (C) 2015 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
#include "uri_constants.h"

QT_BEGIN_NAMESPACE

const QString ROUTING_HOST = QLatin1String("route.ls.hereapi.com");
const QString GEOCODING_HOST = QLatin1String("geocoder.ls.hereapi.com");
const QString REVERSE_GEOCODING_HOST = QLatin1String("reverse.geocoder.ls.hereapi.com");
const QString PLACES_HOST = QLatin1String("places.ls.hereapi.com");
const QString MAP_TILES_HOST = QLatin1String("1-4.base.maps.ls.hereapi.com");
const QString MAP_TILES_HOST_AERIAL = QLatin1String("1-4.aerial.maps.ls.hereapi.com");

QT_END_NAMESPACE
